﻿using System.Collections;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Timers;
using System.Windows;
using System.Windows.Data;
using Infragistics.Windows.Chart;
using System.Windows.Markup;

namespace Chart_Test.Helper
{
    public class SeriesBindingInfo : FrameworkElement
    {
        Timer _Timer = new Timer();

        public SeriesBindingInfo()
        {
            _Timer.Elapsed += new ElapsedEventHandler(_Timer_Elapsed);
            _Timer.AutoReset = false;
        }

        private XamChart targetChart;
        public XamChart TargetChart
        {
            get
            {
                return targetChart;
            }
            set
            {
                targetChart = value;
                //bridge the data context from the parent hierarchy.  
                Binding binding = new Binding("DataContext")
                {
                    Source = targetChart
                };
                SetBinding(DataContextProperty, binding);
                Clear();
                UpdateFromCollection(ItemsSource);
            }
        }

        public static DependencyProperty ItemsSourceProperty = DependencyProperty.Register("ItemsSource", typeof(IEnumerable), typeof(SeriesBindingInfo),
            new FrameworkPropertyMetadata(null, (o, e) =>
            {
                (o as SeriesBindingInfo).OnItemsSourceChanged(e);
            }));

        public IEnumerable ItemsSource
        {
            get
            {
                return (IEnumerable)GetValue(ItemsSourceProperty);
            }
            set
            {
                SetValue(ItemsSourceProperty, value);
            }
        }

        public static DependencyProperty SeriesTemplateProperty = DependencyProperty.Register("SeriesTemplate", typeof(Series), typeof(SeriesBindingInfo),
                new FrameworkPropertyMetadata(null, (o, e) =>
                {
                    (o as SeriesBindingInfo).OnSeriesTemplateChanged(e);
                }));

        public Series SeriesTemplate
        {
            get
            {
                return (Series)GetValue(SeriesTemplateProperty);
            }
            set
            {
                SetValue(SeriesTemplateProperty, value);
            }
        }

        protected void OnItemsSourceChanged(DependencyPropertyChangedEventArgs e)
        {
            Clear();

            if (e.OldValue != null)
            {
                UnRegCollection(e.OldValue as IEnumerable);
            }
            if (e.NewValue != null)
            {
                RegCollection(e.NewValue as IEnumerable);
                UpdateFromCollection(e.NewValue as IEnumerable);
            }
        }

        protected void OnSeriesTemplateChanged(DependencyPropertyChangedEventArgs e)
        {
            Clear();
            UpdateFromCollection(ItemsSource);
        }

        private void Clear()
        {
            if (TargetChart == null)
            {
                return;
            }

            TargetChart.Series.Clear();
        }

        private void UpdateFromCollection(IEnumerable collection)
        {
            if (collection == null)
            {
                return;
            }
            foreach (object item in collection)
            {
                AddItem(item);
            }
        }

        private void AddItem(object item)
        {
            if (SeriesTemplate == null)
            {
                return;
            }

            Series series = new Series();
            if (item is DataSeries && (item as DataSeries).ChartType == ChartType.Point)
            {
                string tempate =
                 @"
                    <DataTemplate 
                        xmlns=""http://schemas.microsoft.com/winfx/2006/xaml/presentation""
                        xmlns:x=""http://schemas.microsoft.com/winfx/2006/xaml"">
                        <Grid>
                            <Ellipse 
                                Width=""" + (item as DataSeries).Thickness + @""" " +
                                @"Height=""" + (item as DataSeries).Thickness + @""" " +
                                @"Fill=""" + (item as DataSeries).Color +  @"""/> " + 
                        @"</Grid>
                    </DataTemplate>
                ";
                
                Marker marker = new Marker
                {
                    UseDataTemplate = true,
                    Format = " ",
                    DataTemplate = XamlReader.Parse(tempate) as DataTemplate, 
                };
                series.Marker = marker;
            }



            series.DataContext = item;

            SetSeriesBindings(series);
            TargetChart.Series.Add(series);
        }

        private void SetSeriesBindings(Series series)
        {
            if (SeriesTemplate == null)
            {
                return;
            }
            foreach (DependencyProperty dp in DependencyPropertyEnumerator.DependencyProperties(typeof(Series)))
            {
                object value;
                BindingExpression be;

                if (DependencyPropertyEnumerator.DependencyPropertyHasBinding(SeriesTemplate, dp, out be))
                {
                    series.SetBinding(dp, be.ParentBinding);
                }
                else if (DependencyPropertyEnumerator.DependencyPropertyHasValue(SeriesTemplate, dp, out value))
                {
                    series.SetValue(dp, value);
                }
            }
        }

        private void RegCollection(IEnumerable collection)
        {
            if (collection is INotifyCollectionChanged)
            {
                (collection as INotifyCollectionChanged).CollectionChanged += new NotifyCollectionChangedEventHandler(SeriesBindingInfo_CollectionChanged);
            }
        }

        private void UnRegCollection(IEnumerable collection)
        {
            if (collection is INotifyCollectionChanged)
            {
                (collection as INotifyCollectionChanged).CollectionChanged -= new NotifyCollectionChangedEventHandler(SeriesBindingInfo_CollectionChanged);
            }
        }

        void SeriesBindingInfo_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (TargetChart == null)
            {
                return;
            }
            Debug.WriteLine("SeriesBindingInfo_CollectionChanged");

            _Timer.Interval = 250;
            _Timer.Enabled = true;
        }

        void _Timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            Debug.WriteLine("_Timer_Elapsed");
            RefreshChart();
        }

        private delegate void Refresher();
        void RefreshChart()
        {
            if (TargetChart.Dispatcher.CheckAccess())
            {
                lock (this)
                {
                    TargetChart.BeginInit();
                    //this could be made more efficient of course.  
                    Clear();
                    UpdateFromCollection(ItemsSource);
                    TargetChart.EndInit();
                }
            }
            else
            {
                TargetChart.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.DataBind, new Refresher(this.RefreshChart));
            }
        }
    }
}
